/**
 * {}
 * @author yutent<yutent.io@gmail.com>
 * @date 2024/03/06 16:34:24
 */

import { Snap, SnapElement, Paper, Fragment } from './svg.js'

let rgNotSpace = /\S+/g,
  rgBadSpace = /[\t\r\n\f]/g,
  rgTrim = /(^\s+|\s+$)/g,
  Str = String,
  elproto = SnapElement.prototype
/*\

     **
     * Adds given class name or list of class names to the element.
     - value (string) class name or space separated list of class names
     **
     = (SnapElement) original element.
    \*/
elproto.addClass = function (value) {
  let classes = Str(value || '').match(rgNotSpace) || [],
    elem = this.node,
    className = elem.className.baseVal,
    curClasses = className.match(rgNotSpace) || [],
    j,
    pos,
    clazz,
    finalValue

  if (classes.length) {
    j = 0
    while ((clazz = classes[j++])) {
      pos = curClasses.indexOf(clazz)
      if (!~pos) {
        curClasses.push(clazz)
      }
    }

    finalValue = curClasses.join(' ')
    if (className != finalValue) {
      elem.className.baseVal = finalValue
    }
  }
  return this
}
/*\

     **
     * Removes given class name or list of class names from the element.
     - value (string) class name or space separated list of class names
     **
     = (SnapElement) original element.
    \*/
elproto.removeClass = function (value) {
  let classes = Str(value || '').match(rgNotSpace) || [],
    elem = this.node,
    className = elem.className.baseVal,
    curClasses = className.match(rgNotSpace) || [],
    j,
    pos,
    clazz,
    finalValue
  if (curClasses.length) {
    j = 0
    while ((clazz = classes[j++])) {
      pos = curClasses.indexOf(clazz)
      if (~pos) {
        curClasses.splice(pos, 1)
      }
    }

    finalValue = curClasses.join(' ')
    if (className != finalValue) {
      elem.className.baseVal = finalValue
    }
  }
  return this
}
/*\

     **
     * Checks if the element has a given class name in the list of class names applied to it.
     - value (string) class name
     **
     = (boolean) `true` if the element has given class
    \*/
elproto.hasClass = function (value) {
  let elem = this.node,
    className = elem.className.baseVal,
    curClasses = className.match(rgNotSpace) || []
  return !!~curClasses.indexOf(value)
}
/*\

     **
     * Add or remove one or more classes from the element, depending on either
     * the class’s presence or the value of the `flag` argument.
     - value (string) class name or space separated list of class names
     - flag (boolean) value to determine whether the class should be added or removed
     **
     = (SnapElement) original element.
    \*/
elproto.toggleClass = function (value, flag) {
  if (flag != null) {
    if (flag) {
      return this.addClass(value)
    } else {
      return this.removeClass(value)
    }
  }
  let classes = (value || '').match(rgNotSpace) || [],
    elem = this.node,
    className = elem.className.baseVal,
    curClasses = className.match(rgNotSpace) || [],
    j,
    pos,
    clazz,
    finalValue
  j = 0
  while ((clazz = classes[j++])) {
    pos = curClasses.indexOf(clazz)
    if (~pos) {
      curClasses.splice(pos, 1)
    } else {
      curClasses.push(clazz)
    }
  }

  finalValue = curClasses.join(' ')
  if (className != finalValue) {
    elem.className.baseVal = finalValue
  }
  return this
}
